#!/usr/bin/perl -w
#
# gwplac.pl
#
# Programme pour obtenir la liste des lieux d'un
# fichier source GeneWeb
#
# (c) Patrick Texier <p.texier@orsennes.com> 07/08/2009
# Programme libre sous licence GNU General Public License (GPL) 2.0
#
# Utilisation en ligne de commande :
# perl gwplac.pl fichier.gw fichier.txt -s
#
# -s est facultatif et permet de remplacer les souligns
# par des espaces si on veut utiliser le fichier de sortie
# pour faire des copier/coller en saisie dans GeneWeb.
#
# 27/10/2009 
# ajout de la rcupration des lieux cbp (non document,
# voir le source gwu.ml)
#
use strict;

my $ligne = '';
my $espace = 0; # remplacement de _ par les espaces ?
my $sourcefile = '';
my $targetfile = '';
# lecture de la ligne de commande
while (exists $ARGV[0]) {
	my $param = shift;
	if ($param eq '-s') {
		$espace = 1;
	}
	elsif ($sourcefile eq '') {
		$sourcefile = $param;
	}
	else {
		$targetfile = $param;
	}
}
# test des paramtres
if ($sourcefile eq '') {
	die "Le nom du fichier source est obligatoire\n"
}
my $cnt = 0;    # compteur de lignes du gw
my $cntplac = 0;	# compteur de lieux
my $cntplacdiff = 0; # compteur de lieux diffrents

my %lieux; # table des lieux

open (SOURCE, $sourcefile) || die "fichier $sourcefile inexistant";
open (TARGET, ">$targetfile") || die "fichier $targetfile impossible  ouvrir";

print "Traitement de $sourcefile vers $targetfile\n";
#
# Recherche du type de fichier en lisant la
# premire ligne
#
my $gedcom = 0;

$ligne = (<SOURCE>);
chomp($ligne);
$cnt++;

# Un gedcom commence par 0 HEAD et n'utilise pas
# les souligns
if ($ligne =~ m/^0 HEAD/) {
	print "gedcom reconnu\n";
	$gedcom = 1;
	$espace = 0;
}
# Un fichier source GeneWeb 5 commence par encoding:
# sinon c'est probablement une version infrieure.
elsif ($ligne =~ m/encoding:/) {
	print "fichier source GeneWeb 5 reconnu\n";
}
else {
	print "fichier non reconnu, traitement source GeneWeb\n";
}

print "Remplacement de _ par une espace\n" if ($espace);
if($gedcom) {
	&trait_gedcom;
}
else {
	&trait;
}

# tri de la table des lieux uniques et
# criture du fichier de sortie

my @tablieux = sort { $a cmp $b } keys %lieux;

foreach my $un_lieu ( @tablieux )
{
	$un_lieu =~ s/_/ /g if ($espace) ;
	print TARGET "$un_lieu\n";
}
close SOURCE;
close TARGET;

print "$cnt lignes, $cntplac lieux dont $cntplacdiff uniques\n";

exit 0;
#
# Extraction des lieux d'un gedcom.
#
# le cas est tellement travial que c'est ais  faire dans Vim :
# :g!/^2 PLAC/d
# :%s/^2 PLAC//
# :%sort u
# :g/^unmarried/d
#
sub trait_gedcom 
{
	while(<SOURCE>) {
		chomp;
		my $lieu;
		$ligne = $_;
		$cnt++;
		#
		# Un lieu gedcom suit 2 PLAC
		#
		if ($ligne =~ m/^2 PLAC (.*)/) {
			$lieu = $1;
			# on ne traite pas 'unmarried'
			# des couples non maris GeneWeb
			if ($lieu ne 'unmarried') {
				$cntplac++;
				&trait_lieu($lieu);
			}
		}
	}
}
#
# Extraction des lieux d'un source geneweb
#
sub trait
{
	while (<SOURCE>) {
		chomp;
		my $lieu;
		$ligne = $_;
		$cnt++;
		# recherche des lieux
		# on recherche les six codes qui correspondent
		# aux lieux dans un gw : #mp #bp #pp #dp #rp cbp
		# et la valeur correspond aux caractres entre
		# les deux espaces suivantes.
		# Il faut traiter les lignes plusieurs fois car
		# l'poux et l'pouse sur sur la mme. Pour les
		# mariages une passe suffirait, c'est esthtique.

		# mariage
		while ($ligne =~ m/(.*)(#mp )(\S*)(.*)/) {
			$ligne = $1 . $4;
			$lieu = $3;
			$cntplac++;
			&trait_lieu($lieu);
		}
		# naissance
		while ($ligne =~ m/(.*)(#bp )(\S*)(.*)/) {
			$ligne = $1 . $4;
			$lieu = $3;
			$cntplac++;
			&trait_lieu($lieu);
		}
		# naissance commune  tous les enfants
		while ($ligne =~ m/(.*)(cbp )(\S*)(.*)/) {
			$ligne = $1 . $4;
			$lieu = $3;
			$cntplac++;
			&trait_lieu($lieu);
		}
		# baptme
		while ($ligne =~ m/(.*)(#pp )(\S*)(.*)/) {
			$ligne = $1 . $4;
			$lieu = $3;
			$cntplac++;
			&trait_lieu($lieu);
		}
		# dcs
		while ($ligne =~ m/(.*)(#dp )(\S*)(.*)/) {
			$ligne = $1 . $4;
			$lieu = $3;
			$cntplac++;
			&trait_lieu($lieu);
		}
		# inhumation
		while ($ligne =~ m/(.*)(#rp )(\S*)(.*)/) {
			$ligne = $1 . $4;
			$lieu = $3;
			$cntplac++;
			&trait_lieu($lieu);
		}
	}
}
#
# criture des lieux uniques dans une table
#
sub trait_lieu
{
	my $lieu = shift;
	if (not exists $lieux{$lieu}) {
		# on ne se sert pas de la valeur
		$lieux{$lieu} = 1;
		$cntplacdiff++;
	}
}
